@@ -5,7 +5,7 @@
 ;; Author: Thomas Fitzsimmons <fitzsim@fitzsim.org>
 ;; Maintainer: Thomas Fitzsimmons <fitzsim@fitzsim.org>
 ;; Created: 2014-09-19
-;; Version: 0.9.3
+;; Version: 0.9.5
 ;; Keywords: calendar
 ;; Homepage: https://www.fitzsim.org/blog/
 ;; Package-Requires: ((emacs "24.1") (fsm "0.2.1") (soap-client "3.2.0") (url-http-ntlm "2.0.4") (nadvice "0.3"))
@@ -67,10 +67,6 @@
 ;; the main domain, e.g., autodiscover.domain.com.  Autodiscovery will
 ;; work in that case.
 
-;; Excorporate must be loaded before any other package that requires
-;; `soap-client'.  The version of `soap-client' that Excorporate
-;; bundles is backward compatible.
-
 ;; Acknowledgments:
 
 ;; Alexandru Harsanyi <AlexHarsanyi@gmail.com> provided help and
@@ -122,6 +118,7 @@
 (require 'fsm)
 (require 'excorporate-calendar)
 (require 'org)
+(require 'excorporate-time-zones)
 
 (defgroup excorporate nil
   "Exchange support."
@@ -131,10 +128,10 @@
 
 (defcustom excorporate-update-diary t
   "If non-nil, Excorporate will add entries to Emacs's diary.
-See also `org-agenda-include-diary' to include retreived entries
+See also `org-agenda-include-diary' to include retrieved entries
 in Org's agenda view.
 
-`excorporate-update-diary' affects the behaviour of `excorporate'
+`excorporate-update-diary' affects the behavior of `excorporate'
 just after a server connection is established.  Changes to this
 variable do not take effect unless `excorporate' is re-run.  If
 one wants to disable or enable Excorporate diary support
@@ -278,6 +275,9 @@ the FSM should transition to on success."
 				      (format "Failed to retrieve %s"
 					      import-url))
 			   (fsm-send fsm :unrecoverable-error))
+		       ;; Here (get-buffer-process (current-buffer))
+		       ;; is nil, so we can't track network processes
+		       ;; to be used in exco-disconnect.
 		       ;; Success, parse WSDL.
 		       (plist-put state-data :retrying nil)
 		       (setf (soap-wsdl-xmlschema-imports wsdl) imports)
@@ -661,6 +661,28 @@ use the `cdr' of the pair as the service URL."
 	(fsm-send fsm :retrieve-xml))
       nil)))
 
+(defun exco-select-connection-identifier ()
+  "Return a connection identifier.
+Return the sole connection if only one exists, or prompt the user
+if more than one connection exists.  Return nil if the user
+provides a null response"
+  (exco--ensure-connection)
+  (if (= (length exco--connection-identifiers) 1)
+      (car exco--connection-identifiers)
+    (let* ((strings (mapcar (lambda (object)
+			      (format "%s" object))
+			    exco--connection-identifiers))
+	   (value (completing-read "Excorporate connection: "
+				   strings nil t)))
+      (unless (equal value "")
+	(let ((position (catch 'index
+			  (let ((index 0))
+			    (dolist (string strings)
+			      (when (equal value string)
+				(throw 'index index))
+			      (setq index (1+ index)))))))
+	  (nth position exco--connection-identifiers))))))
+
 (defun exco-operate (identifier name arguments callback)
   "Execute a service operation asynchronously.
 IDENTIFIER is the connection identifier.  Execute operation NAME
@@ -691,6 +713,22 @@ Examples are \"Exchange2010\", \"Exchange2010_SP1\",
   "Disconnect from a web service.
 IDENTIFIER is the mail address used to look up the connection."
   (exco--with-fsm identifier
+    ;; There does not seem to be a way to track network processes via
+    ;; URL callbacks, so search the process list and close matching
+    ;; connections.
+    (let* ((url (plist-get (fsm-get-state-data fsm) :service-url))
+	   (host (url-host (url-generic-parse-url url))))
+      (dolist (process (process-list))
+	(let* ((contact (process-contact process t))
+	       (process-name (plist-get contact :name))
+	       (process-host (plist-get contact :host))
+	       (process-port (plist-get contact :service)))
+	  (when (and (equal process-name host)
+		     (equal process-host host)
+		     (equal process-port 443))
+	    (delete-process process)))))
+    (let ((process (plist-get (fsm-get-state-data fsm) :process)))
+      (when process (delete-process process)))
     (setq exco--connection-identifiers
 	  (delete identifier exco--connection-identifiers))
     (remhash identifier exco--connections))
@@ -753,6 +791,7 @@ creation."
        (CalendarItem
 	(Subject . ,subject)
 	(Body (BodyType . "Text") ,body)
+	(StartTimeZone (Id . ,(exco-time-zone)))
         (Start . ,(exco-format-date-time start))
         (End . ,(exco-format-date-time end))
         (Location . ,location)
@@ -831,6 +870,7 @@ appointment creation."
 		    (CalendarItem
                      (Subject . ,subject)
 		     (Body (BodyType . "Text") ,body)
+		     (StartTimeZone (Id . ,(exco-time-zone)))
                      (Start . ,(exco-format-date-time start))
                      (End . ,(exco-format-date-time end)))))
 		  nil nil nil nil)
@@ -945,6 +985,10 @@ On each iteration, ITEM is set, and FORMS are run."
 					 (split-string cc-invitees ";"))))
 	    (item-identifier (assoc 'ItemId ,item))
 	    (organizer-structure (assoc 'Organizer ,item)))
+       ;; Silence byte compiler if any of these are unused.
+       (ignore subject start start-internal end end-internal location
+	       to-invitees main-invitees cc-invitees optional-invitees
+	       item-identifier organizer-structure)
        ,@forms)))
 
 (defun exco-calendar-item-with-details-iterate (identifier
@@ -1210,7 +1254,7 @@ ARGUMENT is the prefix argument."
 	   (identifier
 	    (if (y-or-n-p ask-2)
 		mail
-	      (cons mail(completing-read ask-3 (list url) nil nil url)))))
+	      (cons mail (completing-read ask-3 (list url) nil nil url)))))
       (exco-connect identifier)))
    ((exco--string-or-string-pair-p excorporate-configuration)
     ;; A single string or a single pair.
@@ -1224,6 +1268,15 @@ ARGUMENT is the prefix argument."
    (t
     (error "Excorporate: Invalid configuration"))))
 
+(defun excorporate-disconnect ()
+  "Disconnect a server connection."
+  (interactive)
+  (exco--ensure-connection)
+  (let ((identifier (exco-select-connection-identifier)))
+    (when identifier
+      (exco-disconnect identifier)
+      (message "Excorporate: Disconnected %s" identifier))))
+
 (provide 'excorporate)
 
 ;;; excorporate.el ends here
